@@ -7,19 +7,20 @@
 /*
  Used by MatCreateSubMatrices_MPIXAIJ_Local()
 */
-typedef struct { /* used by MatCreateSubMatrices_MPIAIJ_SingleIS_Local() and MatCreateSubMatrices_MPIAIJ_Local */
-  PetscInt   id; /* index of submats, only submats[0] is responsible for deleting some arrays below */
-  PetscInt   nrqs, nrqr;
-  PetscInt **rbuf1, **rbuf2, **rbuf3, **sbuf1, **sbuf2;
-  PetscInt **ptr;
-  PetscInt  *tmp;
-  PetscInt  *ctr;
-  PetscInt  *pa; /* proc array */
-  PetscInt  *req_size, *req_source1, *req_source2;
-  PetscBool  allcolumns, allrows;
-  PetscBool  singleis;
-  PetscInt  *row2proc; /* row to proc map */
-  PetscInt   nstages;
+typedef struct {   /* used by MatCreateSubMatrices_MPIAIJ_SingleIS_Local() and MatCreateSubMatrices_MPIAIJ_Local */
+  PetscInt     id; /* index of submats, only submats[0] is responsible for deleting some arrays below */
+  PetscMPIInt  nrqs, nrqr;
+  PetscInt   **rbuf1, **rbuf2, **rbuf3, **sbuf1, **sbuf2;
+  PetscInt   **ptr;
+  PetscInt    *tmp;
+  PetscInt    *ctr;
+  PetscMPIInt *pa; /* process array */
+  PetscInt    *req_size;
+  PetscMPIInt *req_source1, *req_source2;
+  PetscBool    allcolumns, allrows;
+  PetscBool    singleis;
+  PetscMPIInt *row2proc; /* row to process (MPI rank) map */
+  PetscInt     nstages;
 #if defined(PETSC_USE_CTABLE)
   PetscHMapI cmap, rmap;
   PetscInt  *cmap_loc, *rmap_loc;
@@ -44,19 +45,18 @@ typedef struct {
     Struct header shared by SeqAIJ, SeqBAIJ and SeqSBAIJ matrix formats
 */
 #define SEQAIJHEADER(datatype) \
-  PetscBool         roworiented;  /* if true, row-oriented input, default */ \
-  PetscInt          nonew;        /* 1 don't add new nonzeros, -1 generate error on new */ \
-  PetscInt          nounused;     /* -1 generate error on unused space */ \
-  PetscBool         singlemalloc; /* if true a, i, and j have been obtained with one big malloc */ \
-  PetscInt          maxnz;        /* allocated nonzeros */ \
-  PetscInt         *imax;         /* maximum space allocated for each row */ \
-  PetscInt         *ilen;         /* actual length of each row */ \
-  PetscInt         *ipre;         /* space preallocated for each row by user */ \
+  PetscBool         roworiented; /* if true, row-oriented input, default */ \
+  PetscInt          nonew;       /* 1 don't add new nonzeros, -1 generate error on new */ \
+  PetscInt          nounused;    /* -1 generate error on unused space */ \
+  PetscInt          maxnz;       /* allocated nonzeros */ \
+  PetscInt         *imax;        /* maximum space allocated for each row */ \
+  PetscInt         *ilen;        /* actual length of each row */ \
+  PetscInt         *ipre;        /* space preallocated for each row by user */ \
   PetscBool         free_imax_ilen; \
   PetscInt          reallocs;           /* number of mallocs done during MatSetValues() \
                                         as more values are set than were prealloced */ \
   PetscInt          rmax;               /* max nonzeros in any row */ \
-  PetscBool         keepnonzeropattern; /* keeps matrix structure same in calls to MatZeroRows()*/ \
+  PetscBool         keepnonzeropattern; /* keeps matrix nonzero structure same in calls to MatZeroRows()*/ \
   PetscBool         ignorezeroentries; \
   PetscBool         free_ij;       /* free the column indices j and row offsets i when the matrix is destroyed */ \
   PetscBool         free_a;        /* free the numerical values when matrix is destroy */ \
@@ -192,6 +192,26 @@ typedef struct {
   } \
   (void)0
 
+static inline PetscErrorCode MatXAIJAllocatea(Mat A, PetscInt nz, PetscScalar **array)
+{
+  Mat_SeqAIJ *a = (Mat_SeqAIJ *)A->data;
+
+  PetscFunctionBegin;
+  PetscCall(PetscShmgetAllocateArray(nz, sizeof(PetscScalar), (void **)array));
+  a->free_a = PETSC_TRUE;
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
+static inline PetscErrorCode MatXAIJDeallocatea(Mat A, PetscScalar **array)
+{
+  Mat_SeqAIJ *a = (Mat_SeqAIJ *)A->data;
+
+  PetscFunctionBegin;
+  if (a->free_a) PetscCall(PetscShmgetDeallocateArray((void **)array));
+  a->free_a = PETSC_FALSE;
+  PetscFunctionReturn(PETSC_SUCCESS);
+}
+
 /*
   Frees the a, i, and j arrays from the XAIJ (AIJ, BAIJ, and SBAIJ) matrix types
 */
@@ -200,13 +220,9 @@ static inline PetscErrorCode MatSeqXAIJFreeAIJ(Mat AA, MatScalar **a, PetscInt *
   Mat_SeqAIJ *A = (Mat_SeqAIJ *)AA->data;
 
   PetscFunctionBegin;
-  if (A->singlemalloc) {
-    PetscCall(PetscFree3(*a, *j, *i));
-  } else {
-    if (A->free_a) PetscCall(PetscFree(*a));
-    if (A->free_ij) PetscCall(PetscFree(*j));
-    if (A->free_ij) PetscCall(PetscFree(*i));
-  }
+  if (A->free_a) PetscCall(PetscShmgetDeallocateArray((void **)a));
+  if (A->free_ij) PetscCall(PetscShmgetDeallocateArray((void **)j));
+  if (A->free_ij) PetscCall(PetscShmgetDeallocateArray((void **)i));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 /*
@@ -216,31 +232,32 @@ static inline PetscErrorCode MatSeqXAIJFreeAIJ(Mat AA, MatScalar **a, PetscInt *
 #define MatSeqXAIJReallocateAIJ(Amat, AM, BS2, NROW, ROW, COL, RMAX, AA, AI, AJ, RP, AP, AIMAX, NONEW, datatype) \
   do { \
     if (NROW >= RMAX) { \
-      Mat_SeqAIJ *Ain = (Mat_SeqAIJ *)Amat->data; \
-      /* there is no extra room in row, therefore enlarge */ \
-      PetscInt  CHUNKSIZE = 15, new_nz = AI[AM] + CHUNKSIZE, len, *new_i = NULL, *new_j = NULL; \
-      datatype *new_a; \
+      Mat_SeqAIJ *Ain       = (Mat_SeqAIJ *)Amat->data; \
+      PetscInt    CHUNKSIZE = 15, new_nz = AI[AM] + CHUNKSIZE, len, *new_i = NULL, *new_j = NULL; \
+      datatype   *new_a; \
 \
       PetscCheck(NONEW != -2, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "New nonzero at (%" PetscInt_FMT ",%" PetscInt_FMT ") caused a malloc. Use MatSetOption(A, MAT_NEW_NONZERO_ALLOCATION_ERR, PETSC_FALSE) to turn off this check", ROW, COL); \
       /* malloc new storage space */ \
-      PetscCall(PetscMalloc3(BS2 *new_nz, &new_a, new_nz, &new_j, AM + 1, &new_i)); \
-\
+      PetscCall(PetscShmgetAllocateArray(BS2 * new_nz, sizeof(PetscScalar), (void **)&new_a)); \
+      PetscCall(PetscShmgetAllocateArray(new_nz, sizeof(PetscInt), (void **)&new_j)); \
+      PetscCall(PetscShmgetAllocateArray(AM + 1, sizeof(PetscInt), (void **)&new_i)); \
+      Ain->free_a  = PETSC_TRUE; \
+      Ain->free_ij = PETSC_TRUE; \
       /* copy over old data into new slots */ \
       for (ii = 0; ii < ROW + 1; ii++) new_i[ii] = AI[ii]; \
       for (ii = ROW + 1; ii < AM + 1; ii++) new_i[ii] = AI[ii] + CHUNKSIZE; \
       PetscCall(PetscArraycpy(new_j, AJ, AI[ROW] + NROW)); \
       len = (new_nz - CHUNKSIZE - AI[ROW] - NROW); \
       PetscCall(PetscArraycpy(new_j + AI[ROW] + NROW + CHUNKSIZE, PetscSafePointerPlusOffset(AJ, AI[ROW] + NROW), len)); \
-      PetscCall(PetscArraycpy(new_a, AA, BS2 *(AI[ROW] + NROW))); \
+      PetscCall(PetscArraycpy(new_a, AA, BS2 * (AI[ROW] + NROW))); \
       PetscCall(PetscArrayzero(new_a + BS2 * (AI[ROW] + NROW), BS2 * CHUNKSIZE)); \
       PetscCall(PetscArraycpy(new_a + BS2 * (AI[ROW] + NROW + CHUNKSIZE), PetscSafePointerPlusOffset(AA, BS2 * (AI[ROW] + NROW)), BS2 * len)); \
       /* free up old matrix storage */ \
       PetscCall(MatSeqXAIJFreeAIJ(A, &Ain->a, &Ain->j, &Ain->i)); \
       AA     = new_a; \
-      Ain->a = (MatScalar *)new_a; \
+      Ain->a = new_a; \
       AI = Ain->i = new_i; \
-      AJ = Ain->j       = new_j; \
-      Ain->singlemalloc = PETSC_TRUE; \
+      AJ = Ain->j = new_j; \
 \
       RP   = AJ + AI[ROW]; \
       AP   = AA + BS2 * AI[ROW]; \
@@ -260,8 +277,10 @@ static inline PetscErrorCode MatSeqXAIJFreeAIJ(Mat AA, MatScalar **a, PetscInt *
 \
       PetscCheck(NONEW != -2, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "New nonzero at (%" PetscInt_FMT ",%" PetscInt_FMT ") caused a malloc. Use MatSetOption(A, MAT_NEW_NONZERO_ALLOCATION_ERR, PETSC_FALSE) to turn off this check", ROW, COL); \
       /* malloc new storage space */ \
-      PetscCall(PetscMalloc1(new_nz, &new_j)); \
-      PetscCall(PetscMalloc1(AM + 1, &new_i)); \
+      PetscCall(PetscShmgetAllocateArray(new_nz, sizeof(PetscInt), (void **)&new_j)); \
+      PetscCall(PetscShmgetAllocateArray(AM + 1, sizeof(PetscInt), (void **)&new_i)); \
+      Ain->free_a  = PETSC_FALSE; \
+      Ain->free_ij = PETSC_TRUE; \
 \
       /* copy over old data into new slots */ \
       for (ii = 0; ii < ROW + 1; ii++) new_i[ii] = AI[ii]; \
@@ -274,9 +293,7 @@ static inline PetscErrorCode MatSeqXAIJFreeAIJ(Mat AA, MatScalar **a, PetscInt *
       PetscCall(MatSeqXAIJFreeAIJ(A, &Ain->a, &Ain->j, &Ain->i)); \
       Ain->a = NULL; \
       AI = Ain->i = new_i; \
-      AJ = Ain->j       = new_j; \
-      Ain->singlemalloc = PETSC_FALSE; \
-      Ain->free_a       = PETSC_FALSE; \
+      AJ = Ain->j = new_j; \
 \
       RP   = AJ + AI[ROW]; \
       RMAX = AIMAX[ROW] = AIMAX[ROW] + CHUNKSIZE; \
@@ -458,6 +475,8 @@ PETSC_INTERN PetscErrorCode MatCreateSubMatrix_SeqAIJ(Mat, IS, IS, PetscInt, Mat
 
 PETSC_INTERN PetscErrorCode MatSetSeqAIJWithArrays_private(MPI_Comm, PetscInt, PetscInt, PetscInt[], PetscInt[], PetscScalar[], MatType, Mat);
 
+PETSC_INTERN PetscErrorCode MatResetPreallocation_SeqAIJ_Private(Mat A, PetscBool *memoryreset);
+
 PETSC_SINGLE_LIBRARY_INTERN PetscErrorCode MatSeqAIJCompactOutExtraColumns_SeqAIJ(Mat, ISLocalToGlobalMapping *);
 
 /*
@@ -584,7 +603,7 @@ PETSC_SINGLE_LIBRARY_INTERN PetscErrorCode MatSeqAIJCompactOutExtraColumns_SeqAI
       if (nnz & 0x1) sum += xv[__i] * r[xi[__i]]; \
     } while (0)
 
-#elif defined(PETSC_USE_AVX512_KERNELS) && defined(PETSC_HAVE_IMMINTRIN_H) && defined(__AVX512F__) && defined(PETSC_USE_REAL_DOUBLE) && !defined(PETSC_USE_COMPLEX) && !defined(PETSC_USE_64BIT_INDICES) && !defined(PETSC_SKIP_IMMINTRIN_H_CUDAWORKAROUND)
+#elif !(defined(__GNUC__) && defined(_OPENMP)) && defined(PETSC_USE_AVX512_KERNELS) && defined(PETSC_HAVE_IMMINTRIN_H) && defined(__AVX512F__) && defined(PETSC_USE_REAL_DOUBLE) && !defined(PETSC_USE_COMPLEX) && !defined(PETSC_USE_64BIT_INDICES) && !defined(PETSC_SKIP_IMMINTRIN_H_CUDAWORKAROUND)
   #define PetscSparseDensePlusDot(sum, r, xv, xi, nnz) PetscSparseDensePlusDot_AVX512_Private(&(sum), (r), (xv), (xi), (nnz))
 
 #else
