@@ -16,7 +16,7 @@ PETSC_EXTERN PetscErrorCode TSAdaptCreate_History(TSAdapt);
 /*@C
   TSAdaptRegister -  adds a TSAdapt implementation
 
-  Not Collective
+  Not Collective, No Fortran Support
 
   Input Parameters:
 + sname    - name of user-defined adaptivity scheme
@@ -106,7 +106,7 @@ PetscErrorCode TSAdaptInitializePackage(void)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TSAdaptSetType - sets the approach used for the error adapter
 
   Logicially Collective
@@ -141,7 +141,7 @@ PetscErrorCode TSAdaptSetType(TSAdapt adapt, TSAdaptType type)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TSAdaptGetType - gets the `TS` adapter method type (as a string).
 
   Not Collective
@@ -173,7 +173,7 @@ PetscErrorCode TSAdaptSetOptionsPrefix(TSAdapt adapt, const char prefix[])
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/*@C
+/*@
   TSAdaptLoad - Loads a TSAdapt that has been stored in binary with `TSAdaptView()`.
 
   Collective
@@ -278,14 +278,14 @@ PetscErrorCode TSAdaptDestroy(TSAdapt *adapt)
   PetscFunctionBegin;
   if (!*adapt) PetscFunctionReturn(PETSC_SUCCESS);
   PetscValidHeaderSpecific(*adapt, TSADAPT_CLASSID, 1);
-  if (--((PetscObject)(*adapt))->refct > 0) {
+  if (--((PetscObject)*adapt)->refct > 0) {
     *adapt = NULL;
     PetscFunctionReturn(PETSC_SUCCESS);
   }
 
   PetscCall(TSAdaptReset(*adapt));
 
-  PetscTryTypeMethod((*adapt), destroy);
+  PetscTryTypeMethod(*adapt, destroy);
   PetscCall(PetscViewerDestroy(&(*adapt)->monitor));
   PetscCall(PetscHeaderDestroy(adapt));
   PetscFunctionReturn(PETSC_SUCCESS);
@@ -390,6 +390,12 @@ PetscErrorCode TSAdaptSetAlwaysAccept(TSAdapt adapt, PetscBool flag)
 
   Level: intermediate
 
+  Note:
+  Use `PETSC_CURRENT` to keep the current value for either parameter
+
+  Fortran Note:
+  Use `PETSC_CURRENT_REAL`
+
 .seealso: [](ch_ts), `TSAdapt`, `TSAdaptGetSafety()`, `TSAdaptChoose()`
 @*/
 PetscErrorCode TSAdaptSetSafety(TSAdapt adapt, PetscReal safety, PetscReal reject_safety)
@@ -398,12 +404,12 @@ PetscErrorCode TSAdaptSetSafety(TSAdapt adapt, PetscReal safety, PetscReal rejec
   PetscValidHeaderSpecific(adapt, TSADAPT_CLASSID, 1);
   PetscValidLogicalCollectiveReal(adapt, safety, 2);
   PetscValidLogicalCollectiveReal(adapt, reject_safety, 3);
-  PetscCheck(safety == (PetscReal)PETSC_DEFAULT || safety >= 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Safety factor %g must be non negative", (double)safety);
-  PetscCheck(safety == (PetscReal)PETSC_DEFAULT || safety <= 1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Safety factor %g must be less than one", (double)safety);
-  PetscCheck(reject_safety == (PetscReal)PETSC_DEFAULT || reject_safety >= 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Reject safety factor %g must be non negative", (double)reject_safety);
-  PetscCheck(reject_safety == (PetscReal)PETSC_DEFAULT || reject_safety <= 1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Reject safety factor %g must be less than one", (double)reject_safety);
-  if (safety != (PetscReal)PETSC_DEFAULT) adapt->safety = safety;
-  if (reject_safety != (PetscReal)PETSC_DEFAULT) adapt->reject_safety = reject_safety;
+  PetscCheck(safety == (PetscReal)PETSC_CURRENT || safety >= 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Safety factor %g must be non negative", (double)safety);
+  PetscCheck(safety == (PetscReal)PETSC_CURRENT || safety <= 1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Safety factor %g must be less than one", (double)safety);
+  PetscCheck(reject_safety == (PetscReal)PETSC_CURRENT || reject_safety >= 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Reject safety factor %g must be non negative", (double)reject_safety);
+  PetscCheck(reject_safety == (PetscReal)PETSC_CURRENT || reject_safety <= 1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Reject safety factor %g must be less than one", (double)reject_safety);
+  if (safety != (PetscReal)PETSC_CURRENT) adapt->safety = safety;
+  if (reject_safety != (PetscReal)PETSC_CURRENT) adapt->reject_safety = reject_safety;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -500,6 +506,12 @@ PetscErrorCode TSAdaptGetMaxIgnore(TSAdapt adapt, PetscReal *max_ignore)
 
   Level: intermediate
 
+  Note:
+  Use `PETSC_CURRENT` to keep the current value for either parameter
+
+  Fortran Note:
+  Use `PETSC_CURRENT_REAL`
+
 .seealso: [](ch_ts), `TSAdapt`, `TSAdaptChoose()`, `TSAdaptGetClip()`, `TSAdaptSetScaleSolveFailed()`
 @*/
 PetscErrorCode TSAdaptSetClip(TSAdapt adapt, PetscReal low, PetscReal high)
@@ -508,11 +520,11 @@ PetscErrorCode TSAdaptSetClip(TSAdapt adapt, PetscReal low, PetscReal high)
   PetscValidHeaderSpecific(adapt, TSADAPT_CLASSID, 1);
   PetscValidLogicalCollectiveReal(adapt, low, 2);
   PetscValidLogicalCollectiveReal(adapt, high, 3);
-  PetscCheck(low == (PetscReal)PETSC_DEFAULT || low >= 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Decrease factor %g must be non negative", (double)low);
-  PetscCheck(low == (PetscReal)PETSC_DEFAULT || low <= 1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Decrease factor %g must be less than one", (double)low);
-  PetscCheck(high == (PetscReal)PETSC_DEFAULT || high >= 1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Increase factor %g must be greater than one", (double)high);
-  if (low != (PetscReal)PETSC_DEFAULT) adapt->clip[0] = low;
-  if (high != (PetscReal)PETSC_DEFAULT) adapt->clip[1] = high;
+  PetscCheck(low == (PetscReal)PETSC_CURRENT || low >= 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Decrease factor %g must be non negative", (double)low);
+  PetscCheck(low == (PetscReal)PETSC_CURRENT || low <= 1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Decrease factor %g must be less than one", (double)low);
+  PetscCheck(high == (PetscReal)PETSC_CURRENT || high >= 1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Increase factor %g must be greater than one", (double)high);
+  if (low != (PetscReal)PETSC_CURRENT) adapt->clip[0] = low;
+  if (high != (PetscReal)PETSC_CURRENT) adapt->clip[1] = high;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -564,9 +576,9 @@ PetscErrorCode TSAdaptSetScaleSolveFailed(TSAdapt adapt, PetscReal scale)
   PetscFunctionBegin;
   PetscValidHeaderSpecific(adapt, TSADAPT_CLASSID, 1);
   PetscValidLogicalCollectiveReal(adapt, scale, 2);
-  PetscCheck(scale == (PetscReal)PETSC_DEFAULT || scale > 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Scale factor %g must be positive", (double)scale);
-  PetscCheck(scale == (PetscReal)PETSC_DEFAULT || scale <= 1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Scale factor %g must be less than one", (double)scale);
-  if (scale != (PetscReal)PETSC_DEFAULT) adapt->scale_solve_failed = scale;
+  PetscCheck(scale > 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Scale factor %g must be positive", (double)scale);
+  PetscCheck(scale <= 1, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Scale factor %g must be less than one", (double)scale);
+  adapt->scale_solve_failed = scale;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -610,6 +622,12 @@ PetscErrorCode TSAdaptGetScaleSolveFailed(TSAdapt adapt, PetscReal *scale)
 
   Level: intermediate
 
+  Note:
+  Use `PETSC_CURRENT` to keep the current value for either parameter
+
+  Fortran Note:
+  Use `PETSC_CURRENT_REAL`
+
 .seealso: [](ch_ts), `TSAdapt`, `TSAdaptGetStepLimits()`, `TSAdaptChoose()`
 @*/
 PetscErrorCode TSAdaptSetStepLimits(TSAdapt adapt, PetscReal hmin, PetscReal hmax)
@@ -618,10 +636,10 @@ PetscErrorCode TSAdaptSetStepLimits(TSAdapt adapt, PetscReal hmin, PetscReal hma
   PetscValidHeaderSpecific(adapt, TSADAPT_CLASSID, 1);
   PetscValidLogicalCollectiveReal(adapt, hmin, 2);
   PetscValidLogicalCollectiveReal(adapt, hmax, 3);
-  PetscCheck(hmin == (PetscReal)PETSC_DEFAULT || hmin >= 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Minimum time step %g must be non negative", (double)hmin);
-  PetscCheck(hmax == (PetscReal)PETSC_DEFAULT || hmax >= 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Minimum time step %g must be non negative", (double)hmax);
-  if (hmin != (PetscReal)PETSC_DEFAULT) adapt->dt_min = hmin;
-  if (hmax != (PetscReal)PETSC_DEFAULT) adapt->dt_max = hmax;
+  PetscCheck(hmin == (PetscReal)PETSC_CURRENT || hmin >= 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Minimum time step %g must be non negative", (double)hmin);
+  PetscCheck(hmax == (PetscReal)PETSC_CURRENT || hmax >= 0, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Minimum time step %g must be non negative", (double)hmax);
+  if (hmin != (PetscReal)PETSC_CURRENT) adapt->dt_min = hmin;
+  if (hmax != (PetscReal)PETSC_CURRENT) adapt->dt_max = hmax;
   hmin = adapt->dt_min;
   hmax = adapt->dt_max;
   PetscCheck(hmax > hmin, PETSC_COMM_SELF, PETSC_ERR_ARG_OUTOFRANGE, "Maximum time step %g must greater than minimum time step %g", (double)hmax, (double)hmin);
@@ -684,7 +702,7 @@ PetscErrorCode TSAdaptGetStepLimits(TSAdapt adapt, PetscReal *hmin, PetscReal *h
 .seealso: [](ch_ts), `TSAdapt`, `TSGetAdapt()`, `TSAdaptSetType()`, `TSAdaptSetAlwaysAccept()`, `TSAdaptSetSafety()`,
           `TSAdaptSetClip()`, `TSAdaptSetScaleSolveFailed()`, `TSAdaptSetStepLimits()`, `TSAdaptSetMonitor()`
 @*/
-PetscErrorCode TSAdaptSetFromOptions(TSAdapt adapt, PetscOptionItems *PetscOptionsObject)
+PetscErrorCode TSAdaptSetFromOptions(TSAdapt adapt, PetscOptionItems PetscOptionsObject)
 {
   char      type[256] = TSADAPTBASIC;
   PetscReal safety, reject_safety, clip[2], scale, hmin, hmax;
@@ -874,10 +892,26 @@ PetscErrorCode TSAdaptChoose(TSAdapt adapt, TS ts, PetscReal h, PetscInt *next_s
   PetscAssertPointer(accept, 6);
   if (next_sc) *next_sc = 0;
 
-  /* Do not mess with adaptivity while handling events*/
-  if (ts->event && ts->event->status != TSEVENT_NONE) {
+  /* Do not mess with adaptivity while handling events */
+  if (ts->event && ts->event->processing) {
     *next_h = h;
     *accept = PETSC_TRUE;
+    if (adapt->monitor) {
+      PetscCall(PetscViewerASCIIAddTab(adapt->monitor, ((PetscObject)adapt)->tablevel));
+
+      if (ts->event->iterctr == 0) {
+        /*
+          An event has been found, now finalising the event processing: performing the 1st and 2nd post-event steps.
+          Entering this if-branch means both these steps (set to either PETSC_DECIDE or numerical value) are managed
+          by the event handler. In this case the 1st post-event step is always accepted, without interference of TSAdapt.
+          Note: if the 2nd post-event step is not managed by the event handler (e.g. given 1st = numerical, 2nd = PETSC_DECIDE),
+          this if-branch is not entered, and TSAdapt may reject/adjust the proposed 1st post-event step.
+        */
+        PetscCall(PetscViewerASCIIPrintf(adapt->monitor, "    TSAdapt does not interfere, step %3" PetscInt_FMT " accepted. Processing post-event steps: 1-st accepted just now, 2-nd yet to come\n", ts->steps));
+      } else PetscCall(PetscViewerASCIIPrintf(adapt->monitor, "    TSAdapt does not interfere, step %3" PetscInt_FMT " accepted. Event handling in progress\n", ts->steps));
+
+      PetscCall(PetscViewerASCIISubtractTab(adapt->monitor, ((PetscObject)adapt)->tablevel));
+    }
     PetscFunctionReturn(PETSC_SUCCESS);
   }
 
@@ -888,28 +922,47 @@ PetscErrorCode TSAdaptChoose(TSAdapt adapt, TS ts, PetscReal h, PetscInt *next_s
 
   if (*accept && ts->exact_final_time == TS_EXACTFINALTIME_MATCHSTEP) {
     /* Increase/reduce step size if end time of next step is close to or overshoots max time */
-    PetscReal t = ts->ptime + ts->time_step, h = *next_h;
-    PetscReal tend = t + h, tmax, hmax;
-    PetscReal a    = (PetscReal)(1.0 + adapt->matchstepfac[0]);
-    PetscReal b    = adapt->matchstepfac[1];
+    PetscReal t   = ts->ptime + ts->time_step, tend, tmax, h1, hmax;
+    PetscReal a   = (PetscReal)(1.0 + adapt->matchstepfac[0]);
+    PetscReal b   = adapt->matchstepfac[1];
+    PetscReal eps = 10 * PETSC_MACHINE_EPSILON;
 
-    if (ts->tspan) {
-      if (PetscIsCloseAtTol(t, ts->tspan->span_times[ts->tspan->spanctr], ts->tspan->reltol * h + ts->tspan->abstol, 0)) /* hit a span time point */
-        if (ts->tspan->spanctr + 1 < ts->tspan->num_span_times) tmax = ts->tspan->span_times[ts->tspan->spanctr + 1];
+    /*
+      Logic in using 'dt_span_cached':
+      1. It always overrides *next_h, except (any of):
+         a) the current step was rejected,
+         b) the adaptor proposed to decrease the next step,
+         c) the adaptor proposed *next_h > dt_span_cached.
+      2. If *next_h was adjusted by eval_times points (or the final point):
+           -- when dt_span_cached is filled (>0), it keeps its value,
+           -- when dt_span_cached is clear (==0), it gets the unadjusted version of *next_h.
+      3. If *next_h was not adjusted as in (2), dt_span_cached is cleared.
+      Note, if a combination (1.b || 1.c) && (3) takes place, this means that
+      dt_span_cached remains unused at the moment of clearing.
+      If (1.a) takes place, dt_span_cached keeps its value.
+      Also, dt_span_cached can be updated by the event handler, see tsevent.c.
+    */
+    if (h <= *next_h && *next_h <= adapt->dt_eval_times_cached) *next_h = adapt->dt_eval_times_cached; /* try employing the cache */
+    h1   = *next_h;
+    tend = t + h1;
+
+    if (ts->eval_times && ts->eval_times->time_point_idx < ts->eval_times->num_time_points) {
+      PetscCheck(ts->eval_times->worktol == 0, PetscObjectComm((PetscObject)adapt), PETSC_ERR_PLIB, "Unexpected state (tspan->worktol != 0) in TSAdaptChoose()");
+      ts->eval_times->worktol = ts->eval_times->reltol * h1 + ts->eval_times->abstol;
+      if (PetscIsCloseAtTol(t, ts->eval_times->time_points[ts->eval_times->time_point_idx], ts->eval_times->worktol, 0)) /* hit a span time point */
+        if (ts->eval_times->time_point_idx + 1 < ts->eval_times->num_time_points) tmax = ts->eval_times->time_points[ts->eval_times->time_point_idx + 1];
         else tmax = ts->max_time; /* hit the last span time point */
-      else tmax = ts->tspan->span_times[ts->tspan->spanctr];
+      else tmax = ts->eval_times->time_points[ts->eval_times->time_point_idx];
     } else tmax = ts->max_time;
+    tmax = PetscMin(tmax, ts->max_time);
     hmax = tmax - t;
+    PetscCheck((hmax > eps) || (PetscAbsReal(hmax) <= eps && PetscIsCloseAtTol(t, ts->max_time, eps, 0)), PetscObjectComm((PetscObject)adapt), PETSC_ERR_PLIB, "Unexpected state: bad hmax in TSAdaptChoose()");
+
     if (t < tmax && tend > tmax) *next_h = hmax;
-    if (t < tmax && tend < tmax && h * b > hmax) *next_h = hmax / 2;
-    if (t < tmax && tend < tmax && h * a > hmax) *next_h = hmax;
-    /* if step size is changed to match a span time point */
-    if (ts->tspan && h != *next_h && !adapt->dt_span_cached) adapt->dt_span_cached = h;
-    /* reset time step after a span time point */
-    if (ts->tspan && h == *next_h && adapt->dt_span_cached && PetscIsCloseAtTol(t, ts->tspan->span_times[ts->tspan->spanctr], ts->tspan->reltol * h + ts->tspan->abstol, 0)) {
-      *next_h               = adapt->dt_span_cached;
-      adapt->dt_span_cached = 0;
-    }
+    if (t < tmax && tend < tmax && h1 * b > hmax) *next_h = hmax / 2;
+    if (t < tmax && tend < tmax && h1 * a > hmax) *next_h = hmax;
+    if (ts->eval_times && h1 != *next_h && !adapt->dt_eval_times_cached) adapt->dt_eval_times_cached = h1; /* cache the step size if it is to be changed    */
+    if (ts->eval_times && h1 == *next_h && adapt->dt_eval_times_cached) adapt->dt_eval_times_cached = 0;   /* clear the cache if the step size is unchanged */
   }
   if (adapt->monitor) {
     const char *sc_name = (scheme < ncandidates) ? adapt->candidates.name[scheme] : "";
@@ -992,7 +1045,7 @@ PetscErrorCode TSAdaptCheckStage(TSAdapt adapt, TS ts, PetscReal t, Vec Y, Petsc
   if (func_accept && snesreason < 0 && !snes_div_func) {
     *accept = PETSC_FALSE;
     PetscCall(PetscInfo(ts, "Step=%" PetscInt_FMT ", nonlinear solve failure: %s\n", ts->steps, SNESConvergedReasons[snesreason]));
-    if (++ts->num_snes_failures >= ts->max_snes_failures && ts->max_snes_failures > 0) {
+    if (++ts->num_snes_failures >= ts->max_snes_failures && ts->max_snes_failures != PETSC_UNLIMITED) {
       ts->reason = TS_DIVERGED_NONLINEAR_SOLVE;
       PetscCall(PetscInfo(ts, "Step=%" PetscInt_FMT ", nonlinear solve failures %" PetscInt_FMT " greater than current TS allowed, stopping solve\n", ts->steps, ts->num_snes_failures));
       if (adapt->monitor) {
@@ -1018,7 +1071,7 @@ PetscErrorCode TSAdaptCheckStage(TSAdapt adapt, TS ts, PetscReal t, Vec Y, Petsc
     }
   }
 
-  if (!(*accept) && !ts->reason) {
+  if (!*accept && !ts->reason) {
     PetscReal dt, new_dt;
     PetscCall(TSGetTimeStep(ts, &dt));
     new_dt = dt * adapt->scale_solve_failed;
@@ -1058,11 +1111,9 @@ PetscErrorCode TSAdaptCreate(MPI_Comm comm, TSAdapt *inadapt)
 
   PetscFunctionBegin;
   PetscAssertPointer(inadapt, 2);
-  *inadapt = NULL;
   PetscCall(TSAdaptInitializePackage());
 
   PetscCall(PetscHeaderCreate(adapt, TSADAPT_CLASSID, "TSAdapt", "Time stepping adaptivity", "TS", comm, TSAdaptDestroy, TSAdaptView));
-
   adapt->always_accept      = PETSC_FALSE;
   adapt->safety             = 0.9;
   adapt->reject_safety      = 0.5;
@@ -1079,7 +1130,6 @@ PetscErrorCode TSAdaptCreate(MPI_Comm comm, TSAdapt *inadapt)
   adapt->matchstepfac[1]             = 2.0;  /* halve last step if it is greater than what remains divided this factor */
   adapt->wnormtype                   = NORM_2;
   adapt->timestepjustdecreased_delay = 0;
-
-  *inadapt = adapt;
+  *inadapt                           = adapt;
   PetscFunctionReturn(PETSC_SUCCESS);
 }
