@@ -179,17 +179,18 @@ static PetscErrorCode TSTheta_SNESSolve(TS ts, Vec b, Vec x)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/* We need to transfer X0 which will be copied into sol_prev */
 static PetscErrorCode TSResizeRegister_Theta(TS ts, PetscBool reg)
 {
-  TS_Theta  *th     = (TS_Theta *)ts->data;
-  const char name[] = "ts:theta:X0";
+  TS_Theta *th = (TS_Theta *)ts->data;
 
   PetscFunctionBegin;
-  if (reg && th->vec_sol_prev) {
-    PetscCall(TSResizeRegisterVec(ts, name, th->X0));
-  } else if (!reg) {
-    PetscCall(TSResizeRetrieveVec(ts, name, &th->X0));
+  if (reg) {
+    PetscCall(TSResizeRegisterVec(ts, "ts:theta:sol_prev", th->vec_sol_prev));
+    PetscCall(TSResizeRegisterVec(ts, "ts:theta:X0", th->X0));
+  } else {
+    PetscCall(TSResizeRetrieveVec(ts, "ts:theta:sol_prev", &th->vec_sol_prev));
+    PetscCall(PetscObjectReference((PetscObject)th->vec_sol_prev));
+    PetscCall(TSResizeRetrieveVec(ts, "ts:theta:X0", &th->X0));
     PetscCall(PetscObjectReference((PetscObject)th->X0));
   }
   PetscFunctionReturn(PETSC_SUCCESS);
@@ -226,13 +227,22 @@ static PetscErrorCode TSStep_Theta(TS ts)
     PetscCall(TSAdaptCheckStage(ts->adapt, ts, th->stage_time, th->X, &stageok));
     if (!stageok) goto reject_step;
 
-    th->status = TS_STEP_PENDING;
     if (th->endpoint) {
       PetscCall(VecCopy(th->X, ts->vec_sol));
     } else {
-      PetscCall(VecAXPBYPCZ(th->Xdot, -th->shift, th->shift, 0, th->X0, th->X));
-      PetscCall(VecAXPY(ts->vec_sol, ts->time_step, th->Xdot));
+      PetscCall(VecAXPBYPCZ(th->Xdot, -th->shift, th->shift, 0, th->X0, th->X)); /* th->Xdot is needed by TSInterpolate_Theta */
+      if (th->Theta == 1.0) PetscCall(VecCopy(th->X, ts->vec_sol));              /* BEULER, stage already checked */
+      else {
+        PetscCall(VecAXPY(ts->vec_sol, ts->time_step, th->Xdot));
+        PetscCall(TSAdaptCheckStage(ts->adapt, ts, ts->ptime + ts->time_step, ts->vec_sol, &stageok));
+        if (!stageok) {
+          PetscCall(VecCopy(th->X0, ts->vec_sol));
+          goto reject_step;
+        }
+      }
     }
+
+    th->status = TS_STEP_PENDING;
     PetscCall(TSAdaptChoose(ts->adapt, ts, ts->time_step, NULL, &next_time_step, &accept));
     th->status = accept ? TS_STEP_COMPLETE : TS_STEP_INCOMPLETE;
     if (!accept) {
@@ -699,9 +709,9 @@ static PetscErrorCode TSEvaluateWLTE_Theta(TS ts, NormType wnormtype, PetscInt *
     PetscReal   a = 1 + h_prev / h;
     PetscScalar scal[3];
     Vec         vecs[3];
-    scal[0] = +1 / a;
-    scal[1] = -1 / (a - 1);
-    scal[2] = +1 / (a * (a - 1));
+    scal[0] = -1 / a;
+    scal[1] = +1 / (a - 1);
+    scal[2] = -1 / (a * (a - 1));
     vecs[0] = X;
     vecs[1] = th->X0;
     vecs[2] = th->vec_sol_prev;
@@ -719,7 +729,6 @@ static PetscErrorCode TSRollBack_Theta(TS ts)
   TS        quadts = ts->quadraturets;
 
   PetscFunctionBegin;
-  PetscCall(VecCopy(th->X0, ts->vec_sol));
   if (quadts && ts->costintegralfwd) PetscCall(VecCopy(th->VecCostIntegral0, quadts->vec_sol));
   th->status = TS_STEP_INCOMPLETE;
   if (ts->mat_sensip) PetscCall(MatCopy(th->MatFwdSensip0, ts->mat_sensip, SAME_NONZERO_PATTERN));
@@ -755,7 +764,7 @@ static PetscErrorCode TSForwardStep_Theta(TS ts)
   if (th->endpoint) { /* 2-stage method*/
     th->shift = 1. / ((th->Theta - 1.) * th->time_step0);
     PetscCall(TSComputeIJacobian(ts, th->ptime0, th->X0, th->Xdot, th->shift, J, Jpre, PETSC_FALSE));
-    PetscCall(MatMatMult(J, ts->mat_sensip, MAT_REUSE_MATRIX, PETSC_DEFAULT, &MatDeltaFwdSensip));
+    PetscCall(MatMatMult(J, ts->mat_sensip, MAT_REUSE_MATRIX, PETSC_DETERMINE, &MatDeltaFwdSensip));
     PetscCall(MatScale(MatDeltaFwdSensip, (th->Theta - 1.) / th->Theta));
 
     /* Add the f_p forcing terms */
@@ -771,7 +780,7 @@ static PetscErrorCode TSForwardStep_Theta(TS ts)
   } else { /* 1-stage method */
     th->shift = 0.0;
     PetscCall(TSComputeIJacobian(ts, th->stage_time, th->X, th->Xdot, th->shift, J, Jpre, PETSC_FALSE));
-    PetscCall(MatMatMult(J, ts->mat_sensip, MAT_REUSE_MATRIX, PETSC_DEFAULT, &MatDeltaFwdSensip));
+    PetscCall(MatMatMult(J, ts->mat_sensip, MAT_REUSE_MATRIX, PETSC_DETERMINE, &MatDeltaFwdSensip));
     PetscCall(MatScale(MatDeltaFwdSensip, -1.));
 
     /* Add the f_p forcing terms */
@@ -801,7 +810,7 @@ static PetscErrorCode TSForwardStep_Theta(TS ts)
     if (quadts && quadts->mat_sensip) {
       PetscCall(TSComputeRHSJacobian(quadts, th->ptime0, th->X0, quadJ, NULL));
       PetscCall(TSComputeRHSJacobianP(quadts, th->ptime0, th->X0, quadJp));
-      PetscCall(MatTransposeMatMult(ts->mat_sensip, quadJ, MAT_REUSE_MATRIX, PETSC_DEFAULT, &th->MatIntegralSensipTemp));
+      PetscCall(MatTransposeMatMult(ts->mat_sensip, quadJ, MAT_REUSE_MATRIX, PETSC_DETERMINE, &th->MatIntegralSensipTemp));
       PetscCall(MatAXPY(th->MatIntegralSensipTemp, 1, quadJp, SAME_NONZERO_PATTERN));
       PetscCall(MatAXPY(quadts->mat_sensip, th->time_step0 * (1. - th->Theta), th->MatIntegralSensipTemp, SAME_NONZERO_PATTERN));
     }
@@ -839,14 +848,14 @@ static PetscErrorCode TSForwardStep_Theta(TS ts)
       PetscCall(MatAXPY(ts->mat_sensip, 1, MatDeltaFwdSensip, SAME_NONZERO_PATTERN)); /* stage sensitivity */
       PetscCall(TSComputeRHSJacobian(quadts, th->stage_time, th->X, quadJ, NULL));
       PetscCall(TSComputeRHSJacobianP(quadts, th->stage_time, th->X, quadJp));
-      PetscCall(MatTransposeMatMult(ts->mat_sensip, quadJ, MAT_REUSE_MATRIX, PETSC_DEFAULT, &th->MatIntegralSensipTemp));
+      PetscCall(MatTransposeMatMult(ts->mat_sensip, quadJ, MAT_REUSE_MATRIX, PETSC_DETERMINE, &th->MatIntegralSensipTemp));
       PetscCall(MatAXPY(th->MatIntegralSensipTemp, 1, quadJp, SAME_NONZERO_PATTERN));
       PetscCall(MatAXPY(quadts->mat_sensip, th->time_step0, th->MatIntegralSensipTemp, SAME_NONZERO_PATTERN));
       PetscCall(MatAXPY(ts->mat_sensip, (1. - th->Theta) / th->Theta, MatDeltaFwdSensip, SAME_NONZERO_PATTERN));
     } else {
       PetscCall(TSComputeRHSJacobian(quadts, th->stage_time, ts->vec_sol, quadJ, NULL));
       PetscCall(TSComputeRHSJacobianP(quadts, th->stage_time, ts->vec_sol, quadJp));
-      PetscCall(MatTransposeMatMult(ts->mat_sensip, quadJ, MAT_REUSE_MATRIX, PETSC_DEFAULT, &th->MatIntegralSensipTemp));
+      PetscCall(MatTransposeMatMult(ts->mat_sensip, quadJ, MAT_REUSE_MATRIX, PETSC_DETERMINE, &th->MatIntegralSensipTemp));
       PetscCall(MatAXPY(th->MatIntegralSensipTemp, 1, quadJp, SAME_NONZERO_PATTERN));
       PetscCall(MatAXPY(quadts->mat_sensip, th->time_step0 * th->Theta, th->MatIntegralSensipTemp, SAME_NONZERO_PATTERN));
     }
@@ -1040,8 +1049,8 @@ static PetscErrorCode TSSetUp_Theta(TS ts)
   PetscCall(TSAdaptCandidatesClear(ts->adapt));
   PetscCall(PetscObjectTypeCompare((PetscObject)ts->adapt, TSADAPTNONE, &match));
   if (!match) {
-    PetscCall(VecDuplicate(ts->vec_sol, &th->vec_sol_prev));
-    PetscCall(VecDuplicate(ts->vec_sol, &th->vec_lte_work));
+    if (!th->vec_sol_prev) PetscCall(VecDuplicate(ts->vec_sol, &th->vec_sol_prev));
+    if (!th->vec_lte_work) PetscCall(VecDuplicate(ts->vec_sol, &th->vec_lte_work));
   }
   PetscCall(TSGetSNES(ts, &ts->snes));
 
@@ -1075,7 +1084,7 @@ static PetscErrorCode TSAdjointSetUp_Theta(TS ts)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-static PetscErrorCode TSSetFromOptions_Theta(TS ts, PetscOptionItems *PetscOptionsObject)
+static PetscErrorCode TSSetFromOptions_Theta(TS ts, PetscOptionItems PetscOptionsObject)
 {
   TS_Theta *th = (TS_Theta *)ts->data;
 
@@ -1145,12 +1154,11 @@ static PetscErrorCode TSThetaSetEndpoint_Theta(TS ts, PetscBool flg)
 #if defined(PETSC_HAVE_COMPLEX)
 static PetscErrorCode TSComputeLinearStability_Theta(TS ts, PetscReal xr, PetscReal xi, PetscReal *yr, PetscReal *yi)
 {
-  PetscComplex    z   = xr + xi * PETSC_i, f;
-  TS_Theta       *th  = (TS_Theta *)ts->data;
-  const PetscReal one = 1.0;
+  PetscComplex z  = xr + xi * PETSC_i, f;
+  TS_Theta    *th = (TS_Theta *)ts->data;
 
   PetscFunctionBegin;
-  f   = (one + (one - th->Theta) * z) / (one - th->Theta * z);
+  f   = (1.0 + (1.0 - th->Theta) * z) / (1.0 - th->Theta * z);
   *yr = PetscRealPartComplex(f);
   *yi = PetscImaginaryPartComplex(f);
   PetscFunctionReturn(PETSC_SUCCESS);
