@@ -27,7 +27,7 @@ typedef struct {
 
 /* ------------------------------------------------------------------------------*/
 
-/*@C
+/*@
   TSPseudoComputeTimeStep - Computes the next timestep for a currently running
   pseudo-timestepping process.
 
@@ -62,7 +62,7 @@ PetscErrorCode TSPseudoComputeTimeStep(TS ts, PetscReal *dt)
 /*@C
   TSPseudoVerifyTimeStepDefault - Default code to verify the quality of the last timestep.
 
-  Collective
+  Collective, No Fortran Support
 
   Input Parameters:
 + ts     - the timestep context
@@ -304,7 +304,7 @@ static PetscErrorCode TSPseudoMonitorDefault(TS ts, PetscInt step, PetscReal pti
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-static PetscErrorCode TSSetFromOptions_Pseudo(TS ts, PetscOptionItems *PetscOptionsObject)
+static PetscErrorCode TSSetFromOptions_Pseudo(TS ts, PetscOptionItems PetscOptionsObject)
 {
   TS_Pseudo  *pseudo = (TS_Pseudo *)ts->data;
   PetscBool   flg    = PETSC_FALSE;
@@ -315,7 +315,7 @@ static PetscErrorCode TSSetFromOptions_Pseudo(TS ts, PetscOptionItems *PetscOpti
   PetscCall(PetscOptionsBool("-ts_monitor_pseudo", "Monitor convergence", "", flg, &flg, NULL));
   if (flg) {
     PetscCall(PetscViewerASCIIOpen(PetscObjectComm((PetscObject)ts), "stdout", &viewer));
-    PetscCall(TSMonitorSet(ts, TSPseudoMonitorDefault, viewer, (PetscErrorCode(*)(void **))PetscViewerDestroy));
+    PetscCall(TSMonitorSet(ts, TSPseudoMonitorDefault, viewer, (PetscCtxDestroyFn *)PetscViewerDestroy));
   }
   flg = pseudo->increment_dt_from_initial_dt;
   PetscCall(PetscOptionsBool("-ts_pseudo_increment_dt_from_initial_dt", "Increase dt as a ratio from original dt", "TSPseudoIncrementDtFromInitialDt", flg, &flg, NULL));
@@ -345,7 +345,6 @@ static PetscErrorCode TSView_Pseudo(TS ts, PetscViewer viewer)
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
-/* ----------------------------------------------------------------------------- */
 /*@C
   TSPseudoSetVerifyTimeStep - Sets a user-defined routine to verify the quality of the
   last timestep.
@@ -376,7 +375,7 @@ PetscErrorCode TSPseudoSetVerifyTimeStep(TS ts, PetscErrorCode (*dt)(TS ts, Vec
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ts, TS_CLASSID, 1);
-  PetscTryMethod(ts, "TSPseudoSetVerifyTimeStep_C", (TS, PetscErrorCode(*)(TS, Vec, void *, PetscReal *, PetscBool *), void *), (ts, dt, ctx));
+  PetscTryMethod(ts, "TSPseudoSetVerifyTimeStep_C", (TS, PetscErrorCode (*)(TS, Vec, void *, PetscReal *, PetscBool *), void *), (ts, dt, ctx));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -489,7 +488,7 @@ PetscErrorCode TSPseudoSetTimeStep(TS ts, PetscErrorCode (*dt)(TS ts, PetscReal
 {
   PetscFunctionBegin;
   PetscValidHeaderSpecific(ts, TS_CLASSID, 1);
-  PetscTryMethod(ts, "TSPseudoSetTimeStep_C", (TS, PetscErrorCode(*)(TS, PetscReal *, void *), void *), (ts, dt, ctx));
+  PetscTryMethod(ts, "TSPseudoSetTimeStep_C", (TS, PetscErrorCode (*)(TS, PetscReal *, void *), void *), (ts, dt, ctx));
   PetscFunctionReturn(PETSC_SUCCESS);
 }
 
@@ -642,7 +641,7 @@ PETSC_EXTERN PetscErrorCode TSCreate_Pseudo(TS ts)
 /*@C
   TSPseudoTimeStepDefault - Default code to compute pseudo-timestepping.  Use with `TSPseudoSetTimeStep()`.
 
-  Collective
+  Collective, No Fortran Support
 
   Input Parameters:
 + ts    - the timestep context
