@@ -22,12 +22,12 @@ if __name__ == '__main__':
     '--with-windows-graphics=0',
     '--with-x=0',
     'CC=clang',
-    'CFLAGS=-O2 -pipe -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector-strong -Wno-incompatible-function-pointer-types',
-    'CPPFLAGS=-D__USE_MINGW_ANSI_STDIO=1',
+    'CFLAGS=-O2 -pipe -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector-strong -Wp,-D__USE_MINGW_ANSI_STDIO=1 -Wno-incompatible-function-pointer-types',
+    'CPPFLAGS=',
     'CXX=clang++',
-    'CXXFLAGS=-O2 -pipe -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector-strong',
+    'CXXFLAGS=-O2 -pipe -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector-strong -Wp,-D__USE_MINGW_ANSI_STDIO=1',
     'FC=',
-    'FFLAGS=-O2 -pipe -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector-strong -I/clangarm64/include',
+    'FFLAGS=-O2 -pipe -Wp,-D_FORTIFY_SOURCE=2 -fstack-protector-strong -Wp,-D__USE_MINGW_ANSI_STDIO=1 -I/clangarm64/include',
     'LDFLAGS= -fopenmp',
     'LIBS=',
     'MAKEFLAGS=-j3',
